# TILE DSL Specification
This document is intended to be an introduction of the TILE DSL. The documentation contains explanination and simple examples describing the features of the language.

The TILE DSL is a simple language which supports both plain JavaScript, and the described language features described below. The language is compiled to JavaScript, before it's executed and run in the TILES infrastructure.

## Table of Contents
1. [Starting point](#starting-point)
2. [TILE Identifier](#tile-identifier)
3. [Me](#me)
3. [Commands](#commands)
4. [Events](#events)
5. [Statements](#statements)
    51. [If](#if)
    51. [Repeat](#repeat)
    51. [Sync](#sync)
    51. [Random](#random)
    51. [Reset](#reset)
6. [Data Sources](#data-sources)    
7. [Example Applications](#example-applications)    


## Starting point

The layout required by a TILE application is as follows:

```js
setup(){
    //my setup
}
main(){
    //my application logic
}
```

The `setup` method is optional, and can be removed if it's not required by the application:

```js
main(){
    //my application logic
}
```

Within the setup block, only options for [data sources](#data-sources) are permitted. Please refer to each data-source for available options. The setup block initializes the applications before running the application logic residing within the main block.





## TILE Identifier
A TILE identifier is a reference to a TILE device registered to your user. The reference to a TILE device is done through a keyword: `TILE`, followed by the name of the TILE device. The identifier can only be used with [events](#events) and [commands](commands). Referring to a TILE identifier is accomplished by the following, where `alpha` is the name of the TILE device which should be referenced:

```js
TILE.alpha
```

Retrieving all registered TILES is accomplished through the `TILES` keyword. This will return a nested object, consisting of key and value pairs. The registered name will be the key, and the TILE device, represented as an object, as the value. Each TILE device consist of the following key-value pairs as well: `id`, `active`, `name`, and `state`. The `id` stores the TILE device MAC-address, `active` referes to whether the TILE devices should be made accessible in the TILES infrastructure, `name` the chosen reference name of the TILE device, and `state` contains the latest `event` or `command` retrieved from/to the device. 

All TILES can also be stored within variable for later referencing:

```js
var myTiles = TILES;
```
    
The variable `myTiles` will now be a reference to all your TILE devices, containing the following structure:

```js
{
    alpha:{
        id:"123",
        active:true,
        name:"alpha",
        state: {name:'led', properties:['on']}
    },
    beta:{
        ...
    },
    cita:{
        ...
    }
}
```

Accessing the state of `alpha`

```js
//will now contain the object: {name:'led', properties:['on']}
var alphaState = TILES.alpha.state;

``` 

# Me variable     

Accessing your stored user details are available through the `Me` keyword. Both the name and id are accessible from the DSL language.

```js
Me.name; //access name
Me.id; // access id

var myId=Me.id; //store your id in a variable
``` 

## Commands
Controlling the output interfaces on a TILE device is accomplished by a command. The following output interfaces are currently supported to control: `led`, `speaker`, and `vibrate`.

Each output interface type have their own methods, corresponding to the output interface ability. 

An led for a given TILE device can be turned on, turned off, or set to repeatedly be turned on and off. The default color for the led is `blue`. Supported colors are: `red`, `green`, `blue`, and `white`.  


```js
TILE.alpha.led; //default to turning on the led with a blue color
TILE.alpha.led.on; // default to turn on with a blue color
TILE.alpha.led.on.green;//turn on led with a green color
TILE.alpha.led.off;//turn off the led
TILE.alpha.blink;//repeatedly turn on and off with default color
TILE.alpha.blink.green;//repeatedly turn on and off with a green color
```

A speaker support start playing a sound, set it to pause, or stop it completely.

```js
TILE.alpha.play; //play sound
TILE.alpha.pause; //pause play
TILE.alpha.stop; //stop speaker
```

Vibrate can either be turned on or off for a device

```js
TILE.alpha.vibrate; //default to on
TILE.alpha.vibrate.on; //start vibrating
TILE.alpha.vibrate.off; //stop vibrating
```



A variable referencing a TILE device can also be used with the output interface commands:

```js
var alpha=TILE.alpha;
alpha.led.on.green; //turn led on with a green color for device alpha
```

## Events

Listening for events triggered on devices are accomplished by an event. Events can only be used within [statements](#statements), as it requires some logic to be executed when an event is triggered. You have multiple ways of interacting with your TILE device:

* tapped
    * single
    * double
    * hold 
* tilted
    * left
    * right
    * updown
* rotated
    * clock
    * counter
* shaked
    * horizontally
    * vertically
* shifted
    * lift
    * freefall            

Each event includes a more detailed event, for example, tapping the TILE device, can either trigger a `single`, `double` or `hold` event.

```js
TILE.alpha.tapped  //listen for both single, double and hold event
TILE.alpha.tapped.single  //listen for the single event
TILE.alpha.tapped.double  //listen for the double event
TILE.alpha.tapped.hold  //listen for the hold event
```

Please note these events are not supported by themselves, as a [command](#command), only in [statements](#statements)!


## Statements

Multiple statements are supported by the TILE DSL, which will be described here


### If
An `if` statement consist of an event and a corresponding body, encapsulated within brackets `{` `}` which will be executed when the provided event is triggered. Please refer to the [events](#events) section, to get a list of supported events. 

Execute the code within the brackets when TILE device alpha is single tapped:
```js
if(TILE.alpha.tapped.single){
    ...
}
```
Only a single event is supported for input to the DSL if statement. Operators found in other programming langauges like `&&` and `||` are not supported


A more simple `if` statement is provided as well, which only support an event and a command as input, divided by the `THEN` keyword. Listening for the event will trigger the specified command.

Turn the `led` on for device `beta` when `alpha` is tapped (either single, double, or hold)
```js
if TILE.alpha.tapped then TILE.beta.led.on.red
```



### Repeat
The repeat statement is a simple loop representation for the well-known `for`-loop. 

Repeat body statement 10 times:
```js
var i=0;
repeat(10){
    ++i;
}
//i = 10
```



### Sync

The sync statement is a listener for multiple triggering events. The exact order given is the exact order which will be set up for listening. 

Beta single tapped, alpha double tapped, beta double tapped:
```js
sync(TILE.beta.tapped.single TILE.alpha.tapped.double TILE.beta.tapped.double){
    ...
}
```
Note that each event is divided by a single space charachter. Triggering the input events in a reverse order will not execute the statement!



### Random

The random statement returns a random value from a set of parameter passed to it. It supports both objects, arrays and TILES.

```js
random({"one":"hello", "two":"world"}); //return hello or world
random(["hello","world"]); //return hello or world
random(TILES); //returns a random TILE device
```

The returned value may also be stored with an array:
```js
var tile = random(TILES);
```

### Reset
The reset statement will start the application from the begining of the `main` entry point.

This sample application will turn the led on for the device alpha, wait for it to be tapped, then turn its led off, and start over again.
```js
main(){
    TILE.alpha.led.on;
    
    if(TILE.alpha.tapped){
        TILE.alpha.led.off;
        reset; // return to start
    }
}
```

## Data sources
Data sources are used for communicating with third-party services. Each data source is here explained, with a explanation of which options are required for them to be setup correctly. Commands to third-party services are sending data, while events are referred to as retrieving data, corresponding with TILES events and command handling.

Because the DSL is compiled to JavaScript, each data source event is requiring a following block of code, which will run when the retrieved data is ready for use.

### Twitter
The keyword for refering to the Twitter data-source is `Twitter`. The capital `T` is important for the command to be parsed correctly. 


__Commands:__ 
* `tweet` - For posting a new tweet

```js
Twitter.tweet("this is my first tweet #helloworld");
``` 

__Events:__
* `followers `- retrieve a list of your followers
* `following` - retrieve a list of who you're following

The retrieved data is stored within a variable `twitter`.
```js
Twitter.followers{
    twitter.ids; //array containing the id of each follower
}

Twitter.following{
    twitter.ids; // array containing the id of each follower
}
```




#### Options
Accessing the Twitter API is done using `oauth`, and requires 4 different keys. 

Required setup for twitter:
```js
setup(){
    Twitter.options.consumer_key="my consumer key";
    Twitter.options.consumer_secret="my consumer secret";
    Twitter.options.token="my token";
    Twitter.options.token_secret="my secret token";
}
```
Please refer to the [twitter application management](https://apps.twitter.com/) for creating your keys.



### Facebook

__Commands:__
* `post` - post a status to facebook

```js
Facebook.post("This is my facebook post message");
``` 

__Event:__
* `feed` - retrieves your current feed
* `places` - retrieves your checked-in places

The retrieved data is stored within a variable `facebook`.
```js
Facebook.feed{
    
    for(var i=0;i<facebook.data.length;++i){//loop through all facebook feeds retrieved
        facebook.data[i].story; //story for feed
        facebook.data[i].message; //message for feed
        facebook.data[i].created_time; //feed creation date
    }
}

Facebook.places{
     for(var i=0;i<facebook.data.length;++i){//loop through all facebook feeds retrieved
        facebook.data[i].place; //the place containing it location (lat,lon)
        facebook.data[i].created_time; //tagged place creation date
    }
}
```


#### Options
Required setup for facebook:
```js
setup(){
    Facebook.options.apikey="my apikey";
}
```

Please refer to the [Facebook API](https://developers.facebook.com/) for creating a valid api-key.

### Weather

The weather data source only supports fetching weather data, and includes therefore only events. 

__Events:__
* `current` - fetch current weather
    * Retrieves a large set of data accessible through its available variable: `temperature` `mintemp` `maxtemp` `humidity` `cloudiness` `rain` `snow` `weatherInfo` `sunrise` `sunset` `time` `cityId` `cityName`
* `forecast` - fetch foreact
    * Retrieves a large set of data accessible through its available variable: `numbOfResult` `cityId` `cityName` `getWeatherInfo(<number>)` `avgTemp` `avgMaxTemp` `avgMinTemp` 
* `history` - fetch history weather data
    * Retrieves the same data set as `current`

```js
Weather.current{
    weather.temperature;
    ...  //Refer to the event description above for a definition of the data availables
}
Weather.forecast{
    weather.avgTemp;
    ...  //Refer to the event description above for a definition of the data availables
}
Weather.history{
    weather.temperature;
    ...  //Refer to the event description above for a definition of the data availables
}
```

#### Options
Required setup for OpenWeatherMap:

```js
setup(){
    Weather.options.apikey="my apikey";
}
```

Please refer to the [OpenWeatherMap API](http://openweathermap.org/api) for creating a valid api-key.


### Custom
The CustomHTTP is a more general HTTP client, with no predefined commands or events. Commands can be triggered by the client through its `command` keyword:

```js
CustomHTTP.command("post");//post data to set url
``` 


#### Options

Required setup for a custom HTTP data-source client.

The `CustomHTTP` keyword assigns options to use with a HTTP-request by setting custom data to its option key. The below example shows the structure for assigning data to the custom http. `<myoption>` and `<myvalue>` can here be replaced with anything of your choosing

```js
setup(){
    CustomHTTP.options.<myoption> = <myvalue>
}
```
Attaching an apikey field to the request-data
```js
setup(){
    CustomHTTP.options.apikey = "my api key";
    CustomHTTP.options.url = "my website";
}
```


## Example Applications

The following provides multiple example of TILE applications using the described functionality in the DSL.



__WhackAMole__: Turn a random TILE led on, wait for it to be tapped, then turn the led off, and repeat the application. The `repeat` keyword is automatically assigned to the last block of the TILE application, and do not require it to be set manually. 

```js
main() {
    var tile = random(TILES);
    tile.led;
    if tile.tapped then tile.led.off
}
```

Turn alpha led on, if current temperature is over 14 degrees. The weather is fetched if a sequents of events are triggered by the alpha and beta TILE respectively:
```js
setup(){
Weather.options.apikey="APIKEY";
}
main(){
    sync(TILE.alpha.tapped.double TILE.beta.tapped.double){
        Weather.current{
            if(weather.temperature>14)
                TILE.alpha.led.on.blue;
        }
    }
}
```

Combine multiple data sources:
Here the repeat keywors is required, for the application to be able to listen for all events after its first trigger.
```js
setup(){
    Facebook.options.apikey="myapikey";
    Twitter.options.consumer_key="ckey";
    Twitter.options.consumer_secret="csecret";
    Twitter.options.token="token";
    Twitter.options.token_secret="stoken";
}
main(){
    if TILE.alpha.tapped.double then TILE.beta.led.green
    
    if(TILE.alpha.tapped.single){
        Twitter.tweet("Tap interaction with the alpha device");
        repeat;
    }
    
    if(TILE.beta.tapped){
        Facebook.post("Tap interaction with the beta device");
        repeat;
    }
}
```
