import astring = require('astring');

declare module TileDSL {

    type DslStatement = Array<iDslIf | iDslIfSimple | iRepeat | iSync>;

    type SourceElements = Array<ESTree.Statement | ESTree.FunctionDeclaration | iVariable | iTile | DslStatement>

    export interface Option extends ESTree.Node {
        init: ESTree.Literal
    }

    export interface iMainStatement extends ESTree.Node {
        consequent: ESTree.BlockStatement
    }

    export interface iOptionStatement extends ESTree.Node {
        consequent: Array<Option>
    }

    export interface iDsCommand extends ESTree.Node {
        ds: string, //facebook|twitter|customhttp
        method: string, //post|tweet|command
        value: ESTree.Literal
    }


    /** VARIABLES */
    export interface iVariable extends ESTree.Node {
        option: string
    }

    export interface iFacebookVariable extends ESTree.Node {
        option: "feed" | "places",
        body:ESTree.BlockStatement
    }

    export interface iTwitterVariable extends ESTree.Node {
        option: "followers" | "following",
        body:ESTree.BlockStatement
    }

    export interface iWeatherVariable extends ESTree.Node {
        option: "current" | "forecast" | "history",
        body:ESTree.BlockStatement
    }

    export interface iMeVariable extends ESTree.Node {
        option: "name" | "id" | ""
    }

    export interface iTile extends ESTree.Node {
        name: string//name/id of tile
    }

    /** END VARIABLES */


    /** STATEMENTS */
    export interface iDslIf extends ESTree.Node {
        event: iTileEvent,
        body: SourceElements
    }

    export interface iDslIfSimple extends ESTree.Node {
        event: iTileEvent,
        command: iTileCommand
    }

    export interface iRepeat extends ESTree.Node {
        number: ESTree.Literal, //#loops
        body: SourceElements
    }

    export interface iSync extends ESTree.Node {
        events: Array<iTileEvent>,
        body: SourceElements
    }
    /** END STATEMENTS */


    /** TILE */
    export interface iTileEvent extends ESTree.Node {
        event: string, // type of event (tap|tilt|rotate|shake|vertical_shift)
        tile: iTile,
        param: string, //  type of sub-event (single|double|hold, left|right|updown, clock|counter, horizontally|vertically, lift|freefall)
    }

    export interface iTileCommand extends ESTree.Node {
        tile: iTile,
        command: { type: string, command: string },
        param?: iTileColor | "on" | "off"
    }

    export interface iTileColor extends ESTree.Node {
        color: "red" | "green" | "blue" | "white"
    }

    export interface iRandom extends ESTree.Node {
        random: ESTree.ObjectExpression | ESTree.ArrayExpression | ESTree.Node
    }
    /** END TILE */




    /**
     * Interface describing different types supported by the TILE DSL
     * Each key in the interface corresponds to the types found in the AST
     * 
     * @export
     * @interface TileDsl
     * @extends {astring.DefaultGenerator}
     */
    export interface TileDsl extends astring.DefaultGenerator {
        MainStatement(node: iMainStatement, state: astring.State);
        OptionStatement(node: iOptionStatement, state: astring.State);


        ConsumerSecret(node: Option, state: astring.State);
        ConsumerKey(node: Option, state: astring.State);
        Token(node: Option, state: astring.State);
        TokenSecret(node: Option, state: astring.State);
        FacebookAPI(node: Option, state: astring.State);
        CustomHTTP(node: Option, state: astring.State);
        WeatherAPI(node: Option, state: astring.State);

        facebookvar(node: iFacebookVariable, state: astring.State);
        twittervar(node: iTwitterVariable, state: astring.State);
        weathervar(node: iWeatherVariable, state: astring.State);
        mevar(node: iMeVariable, state: astring.State);
        tile(node: iTile, state: astring.State);
        tiles(node: ESTree.Node, state: astring.State);

        DslIf(node: iDslIf, state: astring.State);
        DslIfSimple(node: iDslIfSimple, state: astring.State);
        Repeat(node: iRepeat, state: astring.State);
        Sync(node: iSync, state: astring.State);
        Random(node: iRandom, state: astring.State);
        Reset(node:ESTree.Node, state:astring.State);
        TileEvent(node: iTileEvent, state: astring.State);
        TileColor(node: iTileColor, state: astring.State);
        TileCommand(node: iTileCommand, state: astring.State);
        DatasourceCommand(node: iDsCommand, state: astring.State);

    }

}


export = TileDSL;
