import HTTPClient = require('../HTTPClient');
import iWeatherInterfaces = require('./iWeatherInterfaces');
import CurrentWeather = require('./CurrentWeather');
import ForecastWeather = require('./ForecastWeather');
import Forecast16Weather = require('./Forecast16Weather');
import ErrorHandler = require('../../../ErrorHandler');


/**
 * API client for openweathermap
 * http://openweathermap.org/api
 */
class Weather {
    private http: HTTPClient = null;

    private baseurl: string = "http://api.openweathermap.org/data/2.5";

    private apiKey: string = null;

    public coordinate: iWeatherInterfaces.iCoordinate = null;
    public cityId: number = null;
    public city: string = null;
    public count: string = null;
    public units: "kelvin" | "metric" | "imperial" = "metric";


    /**
     * Creates an instance of Weather.
     * 
     * @param {string} apiKey to access the api 
     */
    public constructor(apiKey: string) {
        this.apiKey = apiKey;
    }

    private reset() {
        this.coordinate = null;
        this.city = null;
        this.cityId = null;
    }

    /**
     * Generates a query which should be appended to the base url
     * 
     * @private
     * @returns {Object} object query
     */
    private generateQuery(): Object {
        var obj = {};

        if (this.coordinate) {//whether coordinates are set
            obj["lat"] = this.coordinate.lat;
            obj["lon"] = this.coordinate.lon;
        }
        if (this.cityId)//whether a city id is set
            obj["id"] = this.cityId;
        if (this.city)//whether a city query is set
            obj["q"] = this.city;
        if (this.count)//whether a max count is returned
            obj["cnt"] = this.count;
        if (this.units)//whether a preferred units method is set
            obj["units"] = this.units;

        obj["appid"] = this.apiKey;
        return obj;
    }


    /**
     * Validates a query to check if necessary params are given before a http-call is made
     * 
     * @private
     * @returns {boolean} whether necessary params are set or not
     */
    private validateQueryData(): boolean {
        return (this.city == null && this.cityId == null && this.coordinate == null);
    }


    /**
     * Retrieves the current weather for a given location
     * 
     * @param {(data: CurrentWeather) => void} func the function which should retrieve the returned data
     * @returns a CurrentWeather object to param function containing retrieved data
     */
    public current(func: (data: CurrentWeather) => void) {
        if(this.city==null)
        this.city="stadsbygd,norway";
        if (this.validateQueryData())
            return ErrorHandler("Not a valid query for Weather");

        var http = new HTTPClient(this.baseurl + "/weather", "get");
        http.query(this.generateQuery());

        http.exec(function (data: iWeatherInterfaces.iCurrentWeather) {
            if (Weather.isErrorResponse(data))
                return ErrorHandler("Error with weather response");
            func(new CurrentWeather(data));
        });
    }

    /**
     * Retrieves the forecast weather for a given location
     * 
     * @param {(data: ForecastWeather) => void} func the function which should retrieve the returned data
     * @returns a ForecastWeather object to param function containing retrieved data
     */
    public forecast(func: (data: ForecastWeather) => void) {
        if (this.validateQueryData())
            return ErrorHandler("Not a valid query for Weather");


        var http = new HTTPClient(this.baseurl + "/forecast", "get");
        http.query(this.generateQuery());

        http.exec(function (data: iWeatherInterfaces.iForecastWeather) {
            if (Weather.isErrorResponse(data))
                return ErrorHandler("Error with weather response");
            func(new ForecastWeather(data));
        });
    }


    /**
     * Retrieves the forecast weather for a given location 16 days in the future
     * Note: This requires subscription plan for openweathermap
     * 
     * @param {(data: Forecast16Weather) => void} func the function which should retrieve the returned data
     * @returns Forecast16Weather object to param function containing retrieved data
     */
    public forecast16(func: (data: Forecast16Weather) => void) {
        if (this.validateQueryData())
            return ErrorHandler("Not a valid query for Weather");

        var http = new HTTPClient(this.baseurl + "/forecast", "get");
        http.query(this.generateQuery());

        http.exec(function (data: iWeatherInterfaces.iForecast16Weather) {
            if (Weather.isErrorResponse(data))
                return ErrorHandler("Error with weather response");
            func(new Forecast16Weather(data));
        });
    }


    /**
     * Retrieves horuly historic weather data for a given location
     * Note: This requires subscription plan for openweathermap
     * 
     * @param {(data: ForecastWeather) => void} func the function which should retrieve the returned data
     * @returns HistoryWeather object to param function containing retrieved data
     */
    public history(func: (data: ForecastWeather) => void) {
        if (this.validateQueryData())
            return ErrorHandler("Not a valid query for Weather");

        var http = new HTTPClient(this.baseurl + "/history/city", "get");
        http.query(this.generateQuery());

        http.exec(function (data: iWeatherInterfaces.iForecastWeather) {
            if (Weather.isErrorResponse(data))
                return ErrorHandler("Error with weather response");
            func(new ForecastWeather(data));
        });
    }

    public static isErrorResponse(response: JSON | any): boolean {
        if (response["cod"] != undefined && response["cod"] == "404")
            return true;
        return false;
    }
}

export = Weather;