// Interfaces used by OpenWeatherMap
// These interfaces are defined from 
// the api results retrieved by the OpenWeatherMap

export interface iCondition {
    id: number, //Weather condition id
    main: string, //Group of weather parameters (Rain, Snow, Extreme etc.)
    description: string, //Weather condition within the group
    icon: string //Weather icon id
};

export interface iCoordinate {
    lat: number, //City geo location, latitude
    lon: number //City geo location, longitude
}

export interface iInfo {
    temp: number, //Temperature
    pressure?: number, //Atmospheric pressure (on the sea level, if there is no sea_level or grnd_level data), hPa
    humidity: number, //Humidity, %
    temp_min: number, //Minimum temperature at the moment. This is deviation from current temp that is possible for large cities and megalopolises geographically expanded 
    temp_max: number, //Mmaximum temperature at the moment. This is deviation from current temp that is possible for large cities and megalopolises geographically expanded
    sea_level?: number, //Atmospheric pressure on the sea level, hPa
    grnd_level: number //Atmospheric pressure on the ground level, hPa
}

export interface iWind {
    speed: number, //Wind speed. Unit Default: meter/sec, Metric: meter/sec, Imperial: miles/hour.
    deg: number //Wind direction, degrees (meteorological)
}

export interface iClouds {
    all: number //Cloudiness, %
}

export interface iRain {
    _3h: number //Rain volume for the last 3 hours
}

export interface iSnow {
    _3h: number //Snow volume for the last 3 hours
}

export interface iSys {
    type: number, //Internal parameter
    id: number, //Internal parameter
    message: number, //Internal parameter
    country: string, // short name of country Country code (GB, JP etc.)
    sunrise: number, // Sunrise time, unix, UTC
    sunset: number // Sunset time, unix, UTC
}


export interface iCity {
    id: number,//City ID
    name: string, //City name
    coord: iCoordinate
    country: string //Country code (GB, JP etc.)
}

export interface iTemperature {
    day: number,//Day temperature
    min: number,//Min daily temperature
    max: number,// Max daily temperature
    night: number,//Night temperature
    eve: number,//Evening temperature.
    morn: number//Morning temperature
}






// Interfaces used with CurrentAPI
// Call current weather data for one location
// Description is found at:
// http://openweathermap.org/current#JSON
export interface iCurrentWeather {
    coord: iCoordinate,
    weather: Array<iCondition>,

    base: string,//description of station retrieving data, Internal parameter

    main: iInfo,
    wind: iWind,

    clouds?: iClouds,
    rain?: iRain,
    snow?: iSnow,

    dt: number,//Time of data calculation, unix, UTC
    sys: iSys,
    id: number,//id of city
    name: string, //name of place
    cod: number //Internal parameter
}




// Interfaces used with ForecastAPI
// Call 5 day / 3 hour forecast data
// description is foudn at:
// http://openweathermap.org/forecast5#JSON
export interface iForecastWeather {
    code: number, //Internal parameter
    message: number, //Internal parameter
    city: iCity,
    cnt: number //Number of lines returned by this API call
    list: Array<{
        dt: number//Time of data forecasted, unix, UTC
        main: iInfo,
        weather: iCondition
        clouds?: iClouds,
        wind: iWind,
        rain: iRain,
        snow: iSnow
        dt_text: number //Data/time of caluclation, UTC
    }>
}

// Interface used with Forecast16API
// Call 16 day / daily forecast data
// description is found at:
// http://openweathermap.org/forecast16#JSON
export interface iForecast16Weather {
    city: iCity,
    cod: number //internal parameter
    message: number //internal parameter
    cnt: number, //Number of lines returned by this API call
    list: Array<{
        dt: number, //Time of data forecasted
        temp: iTemperature,
        pressure: number, //Atmospheric pressure on the sea level, hPa
        humidity: number, //Humidity, %
        weather: iCondition,
        speed: number, //Wind speed. Unit Default: meter/sec, Metric: meter/sec, Imperial: miles/hour.
        deg: number, //Wind direction, degrees (meteorological)
        clouds: number //Cloudiness, %
    }>
}



// Interface used with HistoryAPI
// Hourly historical data for cities
// description is found at:
// http://openweathermap.org/history#JSON
export interface iHistoryWeather {
    coord: iCoordinate,
    weather: iCondition,
    base: number, //internal parameter
    main: iInfo,
    wind: iWind,
    clouds?: iClouds
    rain?: iRain,
    snow: iSnow,
    dt: number, //Time of data calculation, unix, UTC
    id: number, //City ID,
    name: string, //City name
    cod: number // internal parameter
}