import mqtt = require('mqtt');
import Config = require('../Config');
import Route = require('../Route');
import Tile = require('./User/Tile');
import Event = require('../../Event');
import Command = require('../../Command');
import Events = require('events');
import EventEmitter = require('./Event/EventEmitter');

class MqttClient extends EventEmitter {
    private connection: mqtt.Client = null;
    private route: Route = null;
    private ready: boolean = false;

    /**
     * Creates an instance of MqttClient.
     */
    public constructor() {
        super();
        var self: MqttClient = this;
        this.connection = mqtt.connect(Config.serverName);//connect to server
        this.route = new Route();

        this.connection.on('connect', function() {//listen for connect msg, as mqtt functionality is not available before the client is ready
            self.emit('mqttCon');//fire event when mqtt client is connected
            self.ready = true;
        });

        this.connection.on('message', function(topic: string, msg: Object) {
            var parsedMsg: string | JSON = null;

            try {//try parsing msg to json object
                parsedMsg = JSON.parse(msg.toString());
            }
            catch (e) {
                parsedMsg = msg.toString();//use its string representation if parsing fails
            }

            self.route.parse(topic, [parsedMsg]);//parse the incoming topic, to match against registered routes
        });
    }

    /**
     * Sends a command to a Tile device to control 
     * its interfaces
     * 
     * @param {Tile} tile device that will receive command
     * @param {Command} command to send
     */
    public sendCommand(tile: Tile, command: Command) {
        this.connection.publish("tiles/cmd/" + Config.username + "/" + tile.id, command.toString());
    }

    /**
     * Disconnects the client
     */
    public disconnect() {
        var self: MqttClient = this;
        this.connection.end(true, function() {
            self.emit('mqttDis');
            self.ready = false;
        });
    }

    get isReady(): boolean {
        return this.ready;
    }

    get Route(): Route {
        return this.route;
    }

    get client(): mqtt.Client {
        return this.connection;
    }

    public subscribe(topic: string) {
        this.connection.subscribe(topic);
    }
}

export = MqttClient;