import MqttClient = require("./MqttClient");
import User = require("./User/User");
import Config = require('../Config');
import Events = require('events');
import Event = require('../../Event');
import Command = require('../../Command');
import Tile = require('./User/Tile');
import EventEmitter = require('./Event/EventEmitter');
import HTTPClient = require('./HTTP/HTTPClient');
import Pipe = require('./Event/Pipe');
import EventFilter = require('./Event/EventFilter');

import Facebook = require('./HTTP/Facebook/Facebook');
import Twitter = require('./HTTP/Twitter/Twitter');
import Weather = require('./HTTP/Weather/Weather');
import tile = require('tile');

import LedController = require('./Command/Controller/LedController');
import SpeakerController = require('./Command/Controller/SpeakerController');
import VibrateController = require('./Command/Controller/VibrateController');

abstract class TileDSL extends EventEmitter {
    /**
     * Instance of the mqtt client. Should not be directly accessible to the user-application
     * 
     * @private
     * @type {MqttClient}
     */
    private mqtt: MqttClient = null;

    /**
     * Instance of the current user. Should not be directly accessible to the user-application
     * 
     * @private
     * @type {User}
     */
    private _user: User = null;


    /** @var optionsFunction Function containing option function */
    private optionsFunction: Function = null;

    /** @var mainFunction Function containing option function */
    private mainFunction: Function = null;

    private initialized: boolean = false;

    /** @var debugger Function to call for logging debug information  */
    private debugger: Function = null;

    private pipes: Array<Pipe> = [];

    //variable holding facebook instance
    private _facebook: Facebook = null;

    //variable holding twitter instance
    private _twitter: Twitter = null;

    //variable holding weather instance
    private _weather: Weather = null;
    
    protected led: LedController = null;
    protected speaker: SpeakerController = null;
    protected vibrate: VibrateController = null;

    /**
     * Creates an instance of TileDSL.
     */
    public constructor() {
        super();

        this.mqtt = new MqttClient();
        this.led=new LedController(this.mqtt);
        this.speaker=new SpeakerController(this.mqtt);
        this.vibrate=new VibrateController(this.mqtt);

        var self = this;
        User.findByUsername(Config.username, function (result: User) {
            self._user = result;
            self.emit("ready");//fire event when user is ready
            self.initMqttClient();
        });

        this.mqtt.on("mqttCon", function () {
            self.emit("connected");
        });
    }

    /**
     * Initializes the mqtt client, setting it up for subscribing
     * to events, and handle incoming events
     * 
     * @private
     */
    private initMqttClient() {
        this.mqtt.subscribe("tiles/evt/" + this.user.id + "/#/#");//subscribe to all event topics for current user

        var self = this;
        //handle incoming events of tiles
        this.mqtt.Route.addRoute('tiles/evt/' + this.user.id + '/{id}', function (data: tile.Message, id) {
            //QUICK FIX:
            var prop = [];
            for (let i = 1; i < data.properties.length; ++i)
                prop.push(data.properties[i].replace(/\0/g, ''));
            var tempData = { name: data.properties[0], properties: prop };
            //END QUICK FIX
            
            var event: Event = Event.parse(tempData);//try parsing incoming data

            if (event == null) {
                console.log("Received event is not valid");
                return;
            }

            var tile: Tile = self.user.findTile(id);
            if (tile != null) {//only proceed if the tile is found 
                tile.emit(event.name, event.event);

                self.pipes.forEach(function (pipe, index) {
                    if (pipe.execute({ event: event, tile: tile }))
                        self.emit(pipe.name);
                });
            }
            else console.log("TILE NOT FOUND");
        });
    }

    /**
     * The user code to run when DSL is ready for usage
     */
    public set userFunction(func: Function) {
        var self = this;
        func(function (options, main) {
            self.optionsFunction = options;
            self.mainFunction = main;
        });
    }

    /** 
     * Set debugger to use for logging events
     */
    public set debug(func: Function) {
        this.debugger = func;
    }


    /**
     * Log any output to the attached debugger if set
     * 
     * @protected
     * @param {...string[]} args arguments that should be logged
     */
    protected log(...args) {
        if (this.debugger)//only print if a debugger is set
            this.debugger(args);
    }

    /**
     * Run the user function
     * 
     * @protected
     */
    public run() {
        if (!this.initialized && this.optionsFunction != undefined) {
            this.optionsFunction(this);
            this.initialized = true;
        }
        try {
            this.mainFunction(this);
        }
        catch (e) {
            this.log("Run time error", e);
        }
    }

    /**
     * returns the tiles registered to the user
     * 
     * @readonly
     * @public
     */
    public get tiles() {
        return this._user.objectTiles;
    }

    public get user(): User {
        return this._user;
    }

    /**
     * Generates a new instance of a HTTP client,
     * which can be used to access internet services
     * 
     * @protected
     * @param {string} url to connect to
     * @param {*} method http method to access the site with
     * @returns HTTPClient instance
     */
    protected http(url: string, method: any) {
        return new HTTPClient(url, method);
    }

    protected addPipe(pipe: Pipe) {
        this.pipes.push(pipe);
    }

    protected addFilter(pos: number, eFilter: EventFilter): boolean {
        if (pos < 0 || pos > this.pipes.length)
            return false;
        this.pipes[pos].add(eFilter);
        return true;
    }

    get facebook(): Facebook {
        return this._facebook;
    }

    public initFacebook(apikey: string) {
        this._facebook = new Facebook(apikey);
    }

    get twitter(): Twitter {
        return this._twitter;
    }

    public initTwitter(ckey: string, cSecret: string, token: string, sToken: string) {
        this._twitter = new Twitter(ckey, cSecret, token, sToken);
    }

    get weather(): Weather {
        return this._weather;
    }

    public initWeather(apikey: string) {
        this._weather = new Weather(apikey);
    }

    public initPipe(name: string, ...args: Array<{ tile: Tile, event: tile.iEvent }>) {
        var temp = new Pipe(name);

        args.forEach(arg => {
            if (arg.tile == undefined)
                return;//continue to next item, as tile is not defined

            temp.add(new EventFilter(arg.tile, new Event(arg.event)));
        });
        this.addPipe(temp);
    }

    public random(data: Object | Array<string>) {
        if (data instanceof Array) {
            return data[Math.random() * data.length];
        }
        else if (data instanceof Object) {
            var result = null;
            var count = 0;
            for (var i in data) {
                if (Math.random() < 1 / ++count)
                    result = data[i];
            }
            return result;
        }
        return null;
    }

}

export = TileDSL;