import mongoose = require('mongoose');
import iTile = require('./iTile');
import iUser = require('./iUser');
import ErrorHandler = require('../../ErrorHandler');
import Models = require('../../mongoose/Models');
import Tile = require('./Tile');


class User {
    private document: iUser;
    private oTiles: Object;
    private mapping: Object;

    /**
     * Creates an instance of User.
     * 
     * @param {iUser} document (description)
     */
    constructor(document: iUser) {
        this.document = document;
        this.oTiles = {};
        this.mapping={};
        var self = this;
        this.tiles.forEach(function (tile: iTile, index: number) {
            var identifier=(tile.name!=undefined)?tile.name:tile._id;
            self.oTiles[identifier] = new Tile(tile);
            self.mapping[tile._id] = identifier;
        });
    }

    /**
     * Look up user by username
     * 
     * @static
     * @param {string} username to search for
     * @param {(result: User) => void} cb callback function which will retrieve the returned result on success
     */
    static findByUsername(username: string, cb: (result: User) => void) {
        Models.User.findById(username).populate({
            path: "tiles",
            select: '_id name active state'
        }).exec(function (error: any, user: iUser) {
            if (error)
                return ErrorHandler(error);
            if (!user)
                return ErrorHandler({ code: 101, msg: "User not found" });
            return cb(new User(user));
        });
    }

    /**
     * Get id/username of the user
     * 
     * @readonly
     * @type {string}
     */
    get id(): string {
        return this.document._id;
    }

    /**
     * Retrieves all tiles stored for user
     * 
     * @readonly
     * @type {Array<iTile>}
     */
    get tiles(): Array<iTile> {
        return this.document.tiles;
    }


    /**
     * Retrieves the names for each tile
     * 
     * @readonly
     * @type {Array<string>}
     */
    get tileNames(): Array<string> {
        return Object.keys(this.oTiles);
    }


    /**
     * retrieves an object representation of
     * the tiles stored for the user
     * 
     * @readonly
     * @type {Object}
     */
    get objectTiles(): Object {
        return this.oTiles;
    }

    /**
     * Adds a new tile to the user document
     * and updates the database
     * 
     * @param {iTile} tile interface to add to user
     */
    public addTile(tile: iTile) {
        this.document.tiles.push(tile);
        this.document.save();
    }

    /**
     * Searches for a stored tile by its id.
     * Tiles that are registered to users are
     * only searched for
     * 
     * @param {string} id of tile
     * @returns {iTile} tile Interface
     */
    public findTile(id: string): Tile {
        if (this.mapping[id] != undefined)
            return this.oTiles[this.mapping[id]];
        return null;
    }
}

export = User;
