import mqtt = require('mqtt');
import TileConnection = require('./TileConnection');
import Config = require('./Config');
import crossroads = require('crossroads');
import Event = require('../../Event');

class Client {
    private connection: mqtt.Client = null;
    public static username: string = null;
    public static id: string = null;
    public static tilename: string = null;


    /**
     * Creates an instance of Client.
     * 
     * @param {string} username of connected client
     */
    public constructor(username: string, id: string) {
        if (username == undefined || id == undefined)
            throw new Error("Username or Id not set. Please see documentation");
        crossroads.ignoreState = true;//make sure routes can be matched multiple times 
        Client.username = username;
        Client.id = id;
        Client.tilename = id;
        this.connection = mqtt.connect(Config.SERVER);
        this.initListeners();
    }

    public publishEvent(msg: Event) {
        //TODO: implement tostring method
        //QUICKFIX:
        var prop=[];
        if(msg.name) prop.push(msg.name);
        if(msg.event) prop.push(msg.event);
        if(msg.param) prop.push(msg.event);
        var test={name:Client.tilename,properties:prop};
        //END QUICKFIX
        this.connection.publish("tiles/evt/" + Client.username + "/" + Client.id, JSON.stringify(test));
    }

    /**
     * Initializes listeners which should be called
     * on different events triggered by the mqtt client
     * @see MQTT.js for supported events
     * 
     * @private
     */
    private initListeners() {
        var self = this;

        this.connection.on('connect', function () {
            console.log("client connected");
            self.connection.subscribe("tiles/+/" + Client.username + "/" + Client.id + "/#");
            TileConnection.setConnection(true);
        });

        this.connection.on("close", function () {
            TileConnection.setConnection(false);
        });

        this.connection.on('message', function (topic: string, msg: Object) {
            console.log('received msg', topic);

            var parsedMsg: string | JSON = null;

            //try parse incoming msg to json which should be passed to function
            try {
                parsedMsg = JSON.parse(msg.toString());
            }
            catch (e) {
                parsedMsg = msg.toString();
            }

            crossroads.parse(topic, [parsedMsg]);//parse the incoming topic, to match against registered routes
        });
    }

    /**
     * Add a route which should be parsed against incoming topic
     * 
     * @static
     * @param {string} route to listen for
     * @param {Function} func function to run on match
     */
    private static addRoute(route: string, func: Function) {
        console.log("route add", route);
        crossroads.addRoute(route, func);
    }


    /**
     * Adds new command route which should be triggered when a 
     * command is received. This function is used to abstract
     * common data for each route
     * 
     * @static
     * @param {string} route to listen for
     * @param {Function} func function to run on match
     */
    public static addCmdRoute(route: string, func: Function) {
        console.log("cmd route add");
        this.addRoute("tiles/cmd/" + this.username + "/" + this.id + "/" + route, func);
    }

    /**
    * Adds new event route which should be triggered when a 
    * event is triggered.This function is used to abstract
    * common data for each route
    * 
    * @static
    * @param { string } route to listen for
    * @param { Function } func function to run on match
    */
    public static addEvtRoute(route: string, func: Function) {
        this.addRoute("tiles/evt/" + this.username + "/" + this.id + "/" + route, func);
    }
}

export = Client;