import Client = require('./Client');
import Command = require('../../Command');
import Config = require('./Config');

import Led = require('./output/Led');
import Speaker = require('./output/Speaker');
import Vibrate = require('./output/Vibrate');

import Debugger = require('./Debugger');
import Event = require('../../Event');
import tile = require('tile');

class Device {

    //output doms
    private static led_dom: Element = null;
    private static volume_dom: Element = null;
    private static vibrate_dom: Element = null;

    //input doms
    private static tap_dom: HTMLElement = null;
    private static tilt_dom: HTMLElement = null;
    private static rotate_dom: HTMLElement = null;
    private static shake_dom: HTMLElement = null;
    private static vertical_dom: HTMLElement = null;
    private static proximity_dom: HTMLElement = null;

    //tile doms
    private static tileName_dom: HTMLElement = null;


    private client: Client = null;

    private led: Led = null;
    private speaker: Speaker = null;
    private vibrate: Vibrate = null;

    /**
     * Creates an instance of Device.
     */
    public constructor(username: string, hid: string) {

        Device.led_dom = document.getElementById(Config.LED);
        Device.volume_dom = document.getElementById(Config.VOLUME);
        Device.vibrate_dom = document.getElementById(Config.VIBRATE);

        Device.tap_dom = document.getElementById(Config.TAP);
        Device.tilt_dom = document.getElementById(Config.TILT);
        Device.rotate_dom = document.getElementById(Config.ROTATE);
        Device.shake_dom = document.getElementById(Config.SHAKE);
        Device.vertical_dom = document.getElementById(Config.VERTICALSHIFT);
        Device.proximity_dom = document.getElementById(Config.PROXIMITY);

        Device.tileName_dom = document.getElementById(Config.TILE_NAME_DOM);

        new Debugger();
        this.client = new Client(username, hid);
        this.led = new Led(Device.led_dom);
        this.speaker = new Speaker(Device.volume_dom);
        this.vibrate = new Vibrate(Device.vibrate_dom);
        this.initRoutes();
        this.initListeners();

        Device.tileName_dom.innerText = Client.id;
    }

    /**
     * Initializes topic listeners, and functions to
     * run on a successful topic match
     * 
     * @private
     */
    private initRoutes() {

        var self = this;
        /**
         * when deviec is active/deactivated
         */
        Client.addEvtRoute('active', function (msg: string | JSON) {
            Debugger.log("device", "emulator activated");
        });

        /**
         * when a command is received to emulator
         */
        Client.addCmdRoute('', function (msg: tile.Message) {
            var command: Command = Command.parse(msg);
            if (command == null) {
                Debugger.log("command", "Invalid command received: " + JSON.stringify(msg));
                return;
            }

            switch (command.name) {
                case "led":
                    self.led.parse(command);
                    break;

                case "speaker":
                    self.speaker.parse(command);
                    break;

                case "haptic":
                    self.vibrate.parse(command);
                    break;
            }

        });
    }

    private initListeners() {
        var self = this;
        Device.tap_dom.onclick = function () {
            console.log("TAP CLICKED");
            self.client.publishEvent(new Event({ name: "tap", event: "single" }));
        }
        Device.tilt_dom.onclick = function () {
            self.client.publishEvent(new Event({ name: "tilt", event: "left" }));
        }
        Device.rotate_dom.onclick = function () {
            self.client.publishEvent(new Event({ name: "rotate", event: "clockwise" }));
        }
        Device.shake_dom.onclick = function () {
            self.client.publishEvent(new Event({ name: "shake", event: "horizontally" }));
        }
        Device.vertical_dom.onclick = function () {
            self.client.publishEvent(new Event({ name: "vertical_shift", event: "lift" }));
        }
        Device.proximity_dom.onclick = function () {
            self.client.publishEvent(new Event({ name: "proximity", event: "left" }));
        }
    }
}

export = Device;