import iOutput = require('./iOutput');
import Command = require('../../../Command');
import tile = require('tile');
import Animator = require('../Animator');
/**
 * Class for controling the Led Output for 
 * Tile Emulator
 * 
 * @export
 * @class Led
 * @implements {iOutput}
 */
class Led implements iOutput {
    private dom: Element = null;
    private active: boolean = false;
    private timer = null;
    private command: Command = null;


    /**
     * Creates an instance of Led.
     * 
     * @param {Element} dom (description)
     * @param {boolean} [active=false] (description)
     */
    public constructor(dom: Element, active: boolean = false) {
        this.dom = dom;
        this.active = active;
    }


    /**
     * Toggle the active class on led dom element
     * 
     * @private
     */
    private toggle() {
        if (this.active)
            this.dom.classList.add("active");
        else
            this.dom.classList.remove("active");
    }


    /**
     * Blinks light by calling toggle function
     * @see this.toggle()
     * 
     * @private
     */
    private blink() {
        var self = this;

        //stop timer if already initialized and command for turning it off is received
        if (!this.active || this.timer != null && !this.active) {
            clearInterval(this.timer);
            this.timer = null;
            this.toggle();
        }
        else if (this.timer == null && this.active) {//start timer if a timer is not already registered and command for turning on is received
            this.timer = setInterval(function () {
                //console.log("timeout triggered");
                self.active = (!self.active);
                self.toggle();
            }, this.command.param);
        }

    }

    private fade() {
        var self = this;//this.toggle();

        if (this.dom.classList.contains("active") != this.active)//only fade if the state is changed
            Animator.animate(this.dom.firstElementChild.firstElementChild, "fadeOut", function () {
                self.toggle();
            });
    }


    /**
     * Parse a command and call the appropriate function, as described
     * by the input command
     * 
     * @param {tile.iCommand} command to parse and trigger correct function
     * @returns {boolean} whether command was parsed
     */
    public parse(command: Command): boolean {
        this.command = command;
        // console.log("input command", this.command);
        //this.active = (command.activation == "on") ? true : false;
        this.active = (this.command.event == "on") ? true : false;

        switch (this.command.event) {
            case "on":
            case "off":
                this.toggle();
                return true;

            case "blink":
                this.blink();
                return true;

            case "fade":
                this.fade();
                return true;
        }

        return false;
    }
}

export = Led;
