import Functions = require('../Functions');
import Initializer = require('../Tiles/Initializer');
import Parser = require('../../../DslParser/Parser');
var syntax = require('../../../tiledsl');
var Range = ace.require("ace/range").Range;

declare var hideMetroDialog: Function;

/**
 * TextualEditor controller, initializes the Ace Editor
 * 
 * @class TextualEditorController
 */
class TextualEditorController {


    /**
   * Static variable holding an Ace instance. Made static to be able to use on commands added to the editor
   * @see addCommands
   * 
   * @private
   * @static
   * @type {AceAjax.Editor}
   */
    private static editor: AceAjax.Editor = null;

    private static codeStoreKey = "tiletcode";

    /** Id of charm used in this page */
    private static buildCharmId = "textual-build-charm";

    /** Id of dialog to open when share button pressed from build-charm */
    private static shareDialog = "shareDialog";

    private static parser: Parser = null;

    // Ids of elements defined in build-charm sidebar
    private static buildCharmElements = {
        "compile": "build-compile-code",//widgets
        "share": "build-share-code",
        "save": "build-save-code"
    };

    private static initCode = "setup(){\n//put your options here\n\n}\nmain(){\n//your main code here\n\n}";

    /**
     * Creates an instance of TextualEditorController.
     */
    constructor() {

        this.initEditor();
        this.addCommands();

        //make sure blockly is destructed on hash change (changing route)
        Spa.RouteMapper.runOnNextChange = function () {
            Functions.destroyOptionsMenu();
        }

        Functions.initOptionsMenu(function () {
            Functions.toggleCharm(TextualEditorController.buildCharmId);
        });

        this.initBuildCharm();
        TextualEditorController.parser = new Parser();

    }


    /**
     * Initializes a new Ace editor instance
     * 
     * @private
     */
    private initEditor() {
        TextualEditorController.editor = ace.edit("editor");//id of div
        TextualEditorController.editor.setHighlightActiveLine(true);
        TextualEditorController.editor.setTheme("ace/theme/twilight");
        TextualEditorController.editor.session.setMode("ace/mode/javascript");
        this.setCode(TextualEditorController.retrieveCode());


    }

    /**
     * Adds commands to the editor instance which can be used by the user
     * 
     * @private
     */
    private addCommands() {
        TextualEditorController.editor.commands.addCommand({
            name: 'goToLine',
            bindKey: { win: 'ALT-G', mac: 'Command-G' },
            exec: function (editor) {
                console.log("go to line..");
                TextualEditorController.editor.gotoLine(1);
            }
        });
    }

    private setCode(code: string) {
        //TextualEditorController.editor.getSession().setValue(code);
        if (code !== null)
            TextualEditorController.editor.setValue(code);
    }

    public static storeCode(code: string) {
        store.set(this.codeStoreKey, { code: code, created: new Date() });
    }

    public static retrieveCode(checkDate: Date = null): string {
        var codeObj = store.get(this.codeStoreKey);

        if (codeObj === undefined)
            return this.initCode;

        var date: Date = codeObj.created;
        if (checkDate === null || date > checkDate) {
            var code: string = codeObj.code;

            return code;
        }
        return "Old Code!";
    }

    public static retrieveEditorCode(): string {
        return this.editor.getValue();
    }

    /**
    * Initializes functions to be run when any buttons in the charm menu is pressed
    * 
    * @private
    */
    private initBuildCharm() {

        document.getElementById(TextualEditorController.buildCharmElements.compile).onclick = function () {
            console.log("compiling..");
            try {
                var dsl = TextualEditorController.retrieveEditorCode();
                TextualEditorController.parser.ast = syntax.parse(dsl);
                TextualEditorController.parser.compile();
                console.log(TextualEditorController.parser.code);

                Initializer.tapi.startApp("first", dsl, function (status, message) {
                    if (status) {
                        Functions.openDialog("runDialog");

                        $("#btn_stopCode").on("click", function (e) {
                            Initializer.tapi.stopApp("first", function (status, msg) {
                                console.log("sent del?");
                                if (status)
                                    hideMetroDialog("#runDialog");
                            });

                        });
                    }
                });

            }
            catch (e) {
                var error: PEG.SyntaxError = e;
                console.log(e);
                if (error.location !== null) {

                    /** Highlights lines */
                    /* var range:AceAjax.Range = new Range(error.location.start.line, error.location.start.column, error.location.end.line, error.location.end.column);
                     TextualEditorController.editor.getSession().addMarker(range, "dsl", error.name, true);*/

                    var msg = error.message;
                    var header = error.name + "(ln" + error.location.start.line + ",col" + error.location.start.column + ")";

                    //change status
                    Functions.setStatus(header + ":" + msg, "mif-warning");
                    TextualEditorController.editor.gotoLine(error.location.start.line);

                    //open dialog
                    document.getElementById("compilationHeader").innerText = header;
                    document.getElementById("compilationMsg").innerText = msg;
                    Functions.openDialog("compilationDialog");

                    //add sidebar gutter to line
                    // TextualEditorController.editor.session.addGutterDecoration(error.location.start.line, "ace_error");
                }
            }
        }
        document.getElementById(TextualEditorController.buildCharmElements.share).onclick = function () {
            Functions.toggleCharm(TextualEditorController.buildCharmId, "close");
            Functions.openDialog(TextualEditorController.shareDialog);
        }

        document.getElementById(TextualEditorController.buildCharmElements.save).onclick = function () {

        }
        //TODO: ADD STOP SCRIPT FUNCTIONALITY
    }
}

export = TextualEditorController;